searchNodes=[{"type":"module","title":"Midiex","doc":"This is the main Midiex module. It's built around three basic concepts: Ports: list or count MIDI ports availble (for example, a keyboard or synth) Connections: open or close connections to MIDI ports create a virtual input or output connections so your Elixir application appears as a MIDI device Messages: send or receive messages to and from connections. Examples # List MIDI ports Midiex . ports ( ) # Lists MIDI ports discoverable on your system # [ # %Midiex.MidiPort{ # direction: :input, # name: &quot;IAC Driver Bus 1&quot;, # num: 0, # port_ref: #Reference&lt;0.2239960018.1937899544.176288&gt; # }, # %Midiex.MidiPort{ # direction: :output, # name: &quot;IAC Driver Bus 1&quot;, # num: 0, # port_ref: #Reference&lt;0.2239960018.1937899544.176289&gt; # } # ] # Create a virtual output connection piano = Midiex . create_virtual_output ( &quot;piano&quot; ) # Returns an output connection: # %Midiex.OutConn{ # conn_ref: #Reference&lt;0.1633267383.3718381569.210768&gt;, # name: &quot;piano&quot;, # port_num: 0 # } # Send MIDI messages to a connection # In the message below, the note 60 is equivalent to Middle C and 127 means maximum velocity note_on = &lt;&lt; 0x90 , 60 , 127 &gt;&gt; note_off = &lt;&lt; 0x80 , 60 , 127 &gt;&gt; Midiex . send_msg ( piano , note_on ) :timer . sleep ( 3000 ) # wait three seconds Midiex . send_msg ( piano , note_off ) Livebook tour Also see the introductory tour in LiveBook at /livebook/midiex_notebook.livemd .","ref":"Midiex.html"},{"type":"function","title":"Midiex.close/1","doc":"Closes a MIDI output connection. Accepts as the first parameter either a: MIDI output connection, e.g. a %Midiex.OutConn{} struct List of output connections. Example # Connect to the first output port out_port = Midiex . ports ( :output ) |&gt; List . first ( ) out_conn = Midiex . open ( out_port ) Midiex . close_out_conn ( out_conn ) # Will return :ok if successful","ref":"Midiex.html#close/1"},{"type":"function","title":"Midiex.create_virtual_input/1","doc":"Creates a virtual input port struct. Takes a name as the first parameter. This is only available on platforms that support virtual ports (currently every platform but Windows). Important Even though this creates an input port, beacause it's a virtual port it is listed as an 'output' when querying the OS for available devices. That means other software or devices will discover it and use it as a an output and can send messages to it. It also means it will show as %Midiex.MidiPort{direction: :output} when calling Midiex.ports() . Note that it won't be discoverable until the input port is subscribed. Example # Create a virtual MIDI input by giving it a name. MIDIex will also assign it an input port number (`num`). my_virtual_in = Midiex . create_virtual_input ( &quot;My Virtual Input&quot; ) # This will return a VirtualMidiPort struct in the following format # %Midiex.VirtualMidiPort{direction: :input, name: &quot;My Virtual Input&quot;, num: 1} The %Midiex.VirtualMidiPort{} struct can then be passed to MIDI input port listener functions, such as: Midiex.subscribe(my_virtual_in) If using a Listener GenServer: Midiex.Listener.start_link(port: my_virtual_in) Midiex.Listener.subscribe(listener, my_virtual_in) Likewise, once subscribed to, the virtual input port can be unsubscribed to: Midiex.unsubscribe(my_virtual_in) If using a Listener GenServer: Midiex.Listener.unsubscribe(my_virtual_in)","ref":"Midiex.html#create_virtual_input/1"},{"type":"function","title":"Midiex.create_virtual_output/1","doc":"Creates a virtual output connection. This allows your Elixir application to be seen as a MIDI device. Note this is only available on platforms that support virtual ports (currently every platform but Windows). # Create an output connection called &quot;piano&quot; piano_conn = Midiex . create_virtual_output ( &quot;piano&quot; ) You can send messages to MIDI software or hardware connected to this virtual device in the standard way, e.g.: note_on = &lt;&lt; 0x90 , 60 , 127 &gt;&gt; note_off = &lt;&lt; 0x80 , 60 , 127 &gt;&gt; Midiex . send_msg ( piano , note_on ) :timer . sleep ( 3000 ) # wait three seconds Midiex . send_msg ( piano , note_off ) Important Even though this creates an output port, beacause it's a virtual port it is listed as an 'input' when querying the OS for available devices. That means other software or devices will discover it and use it as a an input, such as to receive messages. It also means it will show as %Midiex.MidiPort{direction: :input} when calling Midiex.ports() .","ref":"Midiex.html#create_virtual_output/1"},{"type":"function","title":"Midiex.hotplug/0","doc":"Ensures that hot-plugging of devices is supported on MacOS. By default on MacOS, Midiex port based functions, such as Midiex.ports() will only list ports visible when the Elixir app was first started. That means devices added or removed afterwards will not be reflected in Midiex.ports() . Important If you'd like functions like Midiex.ports() to reflect the current available ports on your system, such as when plugging or unplugging physical devices, you will need to call the hotplug/0 function before Midiex.ports or Midiex.port_count . You will only need to call hotplug/0 once to enable this mode for the rest of your application's session. This function is similar to the notifications/0 function, expect the calling Elixir process will not receive any MIDI notification messages. If you need to respond to MIDI notification messages, use notifications/0 instead of this function (or use the Midiex.Notifier GenServer) and make sure it has been called before Midiex.ports or Midiex.port_count so hot-plugging is supported.","ref":"Midiex.html#hotplug/0"},{"type":"function","title":"Midiex.notifications/0","doc":"Low-level API for subscribing to MIDI notification messages. Currently only MacOS is supported. The calling process will receive MIDI notification messages. Instead of this function, consider using the Midiex.Notifier GenServer, which will listen to notifications and allow you to create handlers to respond to them. Important To make sure hotplug support works on Midiex.ports() and Midiex.port_count() , make sure this function is called first. It only needs to be called once to enable hotmode support mode for the rest of your application's session. If you want hotplug support but don't need to receive and respond to MIDI notifications, see hotplug/0 instead.","ref":"Midiex.html#notifications/0"},{"type":"function","title":"Midiex.open/1","doc":"Creates a connection to the MIDI port. Accepts one of the following as a parameter: MIDI output port, e.g. a %Midiex.MidiPort{direction: :output} struct List of MIDI output ports. Returns an output connection ( %Midiex.OutConn{) ) or a list of output connections if a list was output ports was given as the first parameter. Example Connect to a single output port # Get the first available MIDI output port out_port = Midiex . ports ( :output ) |&gt; List . first ( ) # Open an output connection out_conn = Midiex . open ( out_port ) # Returns an output connection struct, for example: % Midiex.OutConn { conn_ref : # Reference &lt; 0.3613027763 . 2067398660.163505 &gt; , name : &quot;IAC Driver Bus 1&quot; , port_num : 0 } You can now send messages to the connection: # Send a note on message for D4 Midiex . msg_send ( out_conn , Midiex.Message . note_on ( :D4 ) ) # Let note play for 2 seconds :timer . sleep ( 2000 ) # Send a note off message for D4 Midiex . msg_send ( out_conn , Midiex.Message . note_off ( :D4 ) ) Connect to a list of output ports # Get a list of available output ports out_ports = Midiex . ports ( :output ) # Returns a list of available MIDI output ports, e.g.: [ % Midiex.MidiPort { direction : :output , name : &quot;Arturia MicroFreak&quot; , num : 1 , port_ref : # Reference &lt; 0.3139841870 . 4103995416.58431 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.3139841870 . 4103995416.58432 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;MiniFuse 2&quot; , num : 3 , port_ref : # Reference &lt; 0.3139841870 . 4103995416.58433 &gt; } ] # Connect to each port out_conns = Midiex . open ( out_ports ) # Returns a list of MIDI output connections, e.g.: [ % Midiex.OutConn { conn_ref : # Reference &lt; 0.1633267383 . 3718381569.210768 &gt; , name : &quot;Arturia MicroFreak&quot; , port_num : 1 } , % Midiex.OutConn { conn_ref : # Reference &lt; 0.3139841870 . 4103995416.58432 &gt; , name : &quot;KeyStep Pro&quot; , port_num : 2 } , % Midiex.OutConn { conn_ref : # Reference &lt; 0.3139841870 . 4103995416.58433 &gt; , name : &quot;MiniFuse 2&quot; , port_num : 3 } ]","ref":"Midiex.html#open/1"},{"type":"function","title":"Midiex.port_count/0","doc":"Returns the count of the number of input and output MIDI ports in as a map. Midiex . port_count ( ) # Returns a map in the following format: # %{input: 2, output: 0}","ref":"Midiex.html#port_count/0"},{"type":"function","title":"Midiex.ports/0","doc":"Lists MIDI ports availabile on the system. Midiex . ports ( ) # Returns a list of input or output ports: # [ # %Midiex.MidiPort{ # direction: :input, # name: &quot;Piano&quot;, # num: 0, # port_ref: #Reference&lt;0.249304305.242352152.40090&gt; # }, # %Midiex.MidiPort{ # direction: :input, # name: &quot;Drums&quot;, # num: 1, # port_ref: #Reference&lt;0.249304305.242352152.40091&gt; # } # ]","ref":"Midiex.html#ports/0"},{"type":"function","title":"Midiex.ports/1","doc":"List MIDI ports matching the specified direction (e.g. input or output) Takes an atom as the first parameter representing the direction: :input - lists input ports only :output - lists output ports only. Midiex . ports ( :input ) # Returns a list of input or output MIDI ports: # [ # %Midiex.MidiPort{ # direction: :input, # name: &quot;Piano&quot;, # num: 0, # port_ref: #Reference&lt;0.249304305.242352152.40090&gt; # }, # %Midiex.MidiPort{ # direction: :input, # name: &quot;Drums&quot;, # num: 1, # port_ref: #Reference&lt;0.249304305.242352152.40091&gt; # } # ]","ref":"Midiex.html#ports/1"},{"type":"function","title":"Midiex.ports/2","doc":"Lists MIDI ports matching the name. This can be either a string match or a regex pattern. Optionally takes a direction (:input or :output) can be given. Examples: # Regex examples # List ports containing the word &#39;Arturia&#39; and ignore case Midiex . ports ( ~r/Arturia/i ) # List output ports starting the word &#39;Arturia&#39; and ignore case Midiex . ports ( ~r/^Arturia/i , :output ) # String matching examples # List input ports with the name &#39;KeyStep Pro&#39; Midiex . ports ( &quot;KeyStep Pro&quot; , :input ) # List output ports with the name &#39;Arturia MicroFreak&#39; Midiex . ports ( &quot;Arturia MicroFreak&quot; , :output )","ref":"Midiex.html#ports/2"},{"type":"function","title":"Midiex.send_msg/2","doc":"Sends a binary MIDI message to one or more output connection(s). Takes the following parameters: Output connection: which is an %Midiex.OutConn{} struct or a list of Midiex.OutConn{} structs MIDI message: which is in a binary format, such as &lt;&lt;0x90, 60, 127&gt;&gt; Returns the same output connection or a list of output connections passed to it. This is so you can chain messages together. Example Send a message to single output connection # Connect to the first available MIDI output port out_conn = Midiex . ports ( :output ) |&gt; List . first ( ) |&gt; Midiex . open ( ) # Play the note D3 on all that connection for 3 seconds out_conn |&gt; Midiex . send_msg ( Midiex.Message . note_on ( :D3 ) ) |&gt; tap ( fn _ -&gt; :timer . sleep ( 3000 ) end ) # wait 3 seconds |&gt; Midiex . send_msg ( Midiex.Message . note_off ( :D3 ) ) Send the same message to ALL available outputs # Get a list of output ports and create connections for them out_conns = Midiex . ports ( :output ) |&gt; Midiex . open ( ) # Play the note D3 on all those connections for 3 seconds out_conns |&gt; Midiex . send_msg ( Midiex.Message . note_on ( :D3 ) ) |&gt; tap ( fn _ -&gt; :timer . sleep ( 3000 ) end ) # wait 3 seconds |&gt; Midiex . send_msg ( Midiex.Message . note_off ( :D3 ) )","ref":"Midiex.html#send_msg/2"},{"type":"function","title":"Midiex.subscribe/1","doc":"Low-level API for subscribing to one or more MIDI input ports. This includes ports created via create_virtual_input/1 . The first parameter accepts either: A single %Midiex.MidiPort{direction: :input} or %Midiex.VirtualMidiPort{direction: :input} struct A list of %Midiex.MidiPort{direction: :input} or %Midiex.VirtualMidiPort{direction: :input} structs. The calling process will receive MIDI messages from the ports subscribed to. The source of the message will be differentiated by the input port, but also consider using a different calling process for different inputs if they need to be handled separately. Example # Get a list of MIDI input ports on the system midi_input_ports = Midiex . ports ( :input ) # This function will return a list of ports discovered, e.g.: # [ # %Midiex.MidiPort{ # direction: :input, # name: &quot;Arturia MicroFreak&quot;, # num: 1, # port_ref: #Reference&lt;0.3704955737.2291269656.36760&gt; # }, # %Midiex.MidiPort{ # direction: :input, # name: &quot;KeyStep Pro&quot;, # num: 2, # port_ref: #Reference&lt;0.3704955737.2291269656.36761&gt; # }, # %Midiex.MidiPort{ # direction: :input, # name: &quot;MiniFuse 2&quot;, # num: 3, # port_ref: #Reference&lt;0.3704955737.2291269656.36762&gt; # } # ] # Subscribe to the input ports. The current process will receive MIDI messages. Returns `:ok` if successful. Midiex . subscribe ( midi_input_ports ) You'll need to implement message recieving in your process. Alternative: use a Listener process As an alterantive you can use Midiex.Listener GenServer which subscribes to MIDI input ports and forwards any messages received to event handlers. For the above example, if you wanted to inspect messages coming in on those ports, you could: alias Midiex.Listener # Start a lister for this MIDI input port { :ok , listner } = Listener . start_link ( port : midi_input_ports ) # Create a handler than inspects the MIDI messages received: Listener . add_handler ( listener , fn ( midi_msg ) -&gt; IO . inspect ( midi_msg . data , label : &quot;Msg received&quot; ) end ) # Any messages received will be inspected on the console, e.g.: # Msg received: [130, 84, 0] # Msg received: [146, 60, 43] # Msg received: [130, 60, 0] # Msg received: [146, 72, 34] # Msg received: [130, 72, 0] # Msg received: [146, 76, 40] # Msg received: [130, 76, 0] # Msg received: [146, 64, 47] # Msg received: [130, 64, 0] # Msg received: [146, 84, 30]","ref":"Midiex.html#subscribe/1"},{"type":"function","title":"Midiex.subscribed_ports/0","doc":"Returns a list of ports currently subscribed to using the subscribe/1 function. This will include any vitual input ports currently subscribed to. Example Midiex . subscribed_ports ( ) # Returns of list of subscribed ports, including virtual ports, e.g.: [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.3977715800 . 1277296664.256043 &gt; } , % Midiex.VirtualMidiPort { direction : :input , name : &quot;My Virtual Input&quot; , num : 1 } ]","ref":"Midiex.html#subscribed_ports/0"},{"type":"function","title":"Midiex.unsubscribe/1","doc":"Unsubscribes from recieving MIDI messages from an input port connection. Messages stop as this function releases the OS thread created for listening to the input conection. This function takes as the first parameter one of the following: Port struct: A MIDI input port %Midiex.MidiPort{direction: :input} or %Midiex.VirtualMidiPort{direction: :input} struct List: A list of MIDI input port %Midiex.MidiPort{direction: :input} or %Midiex.VirtualMidiPort{direction: :input} structs Number: A MIDI input port number (this is the integer in the :num key within the %Midiex.MidiPort{} ) (non-virtual ports only) Atom: The atom :all , which will unsubscribe from all MIDI input ports subscribed to, including virtual ports. If you would like to unsubscribe to virtual ports or ones listed on your device by the OS only, use unsubscribe(:all, :virtual) or unsubscribe(:all, :device) instead. Example # Unsubscribe from all inport ports listed on your device&#39;s OS Midiex . unsubscribe ( :all , :device ) # Unsubscribe from all virtual ports you created Midiex . unsubscribe ( :all , :virtual )","ref":"Midiex.html#unsubscribe/1"},{"type":"module","title":"Midiex.Listener","doc":"GenServer for subscribing to MIDI input ports and responding to the MIDI messages ( Midiex.MidiMessage ) received. How this works This GenServer works by: Subscribing to one or more MIDI input ports (using Midiex.subscribe/1 ). For each MIDI input port, Midiex.subscribe/1 will create a new OS thread (in Rust) which establishes a connection to the port and listens to messages. Incoming messages are then forwarded to the calling Elixir process (in this case, the Midiex.Listener process.) A subscription can be established on the start_link/1 or subscribe/2 functions, e.g.: # Get the first MIDI input port input_port = Midiex . ports ( :input ) |&gt; List . first ( ) # Start a lister for this MIDI input port { :ok , listner } = Midiex.Listener . start_link ( port : input_port ) Receieves MIDI messages in the form of a Midiex.MidiMessage struct, and passes it onto one or more Elixir handler functions. The handler takes one parameter representing the MIDI message, e.g.: # Add a simple message handler which inspects each message received: Listener . add_handler ( listener , fn ( midi_msg ) -&gt; IO . inspect ( midi_msg ) end ) Example alias Midiex.Listener # Get the first MIDI input port input_port = Midiex . ports ( :input ) |&gt; List . first ( ) # Start a lister for this MIDI input port { :ok , listner } = Listener . start_link ( port : input_port ) # Create a handler than inspects the MIDI messages received: my_msg_hander = fn ( midi_msg ) -&gt; IO . inspect ( midi_msg , label : &quot;MIDI message&quot; ) end Listener . add_handler ( listener , &amp; my_msg_hander / 1 ) # Stop listening to the input port Listener . unsubscribe ( listner , input_port )","ref":"Midiex.Listener.html"},{"type":"function","title":"Midiex.Listener.add_handler/2","doc":"Add one or more callback function(s) which will recieve and handle MIDI messages. A single callback function or multiple callback functions can be provided in a list. Example # Start your Listener process { :ok , listener } = Listener . start_link ( port : input_port ) # Add a single handler Listener . add_handler ( listener , fn msg -&gt; IO . inspect msg , label : &quot;Inspecting msg&quot; end ) # Add multiple handlers in a list Listener . add_handler ( listener , [ fn msg -&gt; IO . inspect msg , label : &quot;Msg handler 1&quot; end , fn msg -&gt; IO . inspect msg , label : &quot;Msg handler 1&quot; end , ] ) # If you&#39;ve defined your hander function in a module function, pass it the usual way: Listener . add_handler ( listener , &amp; MyModule . function_name / 1 )","ref":"Midiex.Listener.html#add_handler/2"},{"type":"function","title":"Midiex.Listener.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Midiex.Listener.html#child_spec/1"},{"type":"function","title":"Midiex.Listener.get_state/1","doc":"Gets the servers state, returns %Midiex.Listener{} struct.","ref":"Midiex.Listener.html#get_state/1"},{"type":"function","title":"Midiex.Listener.new/1","doc":"Creates a new %Midiex.Server{} struct. Takes an optional keyword list as the first parameter which can be used to populate individual struct keys. The struct holds the following key-values: :port which holds a list of MIDI input ports to listen to. These can be device ports %Midiex.MidiPort{direction: :input} or virtual ports %Midiex.VirtualMidiPort{} :callback which holds a list of functions called when a message is received for an input port. The callback must be of single arity and take it's first parameter a message. See add_handler/2 for an example.","ref":"Midiex.Listener.html#new/1"},{"type":"function","title":"Midiex.Listener.start_link/1","doc":"Start the Midiex.Server GenServer. Takes an optional keyword list as the first parameter which can be used to populate individual %Midiex.Listener{} struct keys. See new/1 for informaton. Examples # Start with no options { :ok , listener } = Midiex.Listener . start_link ( ) # Start, already passing the first available input port to listen to first_port = Midiex . ports ( :input ) |&gt; List . first ( ) { :ok , listener } = Midiex.Listener . start_link ( ports : first_port ) # Start, already passing a list of all input ports available to listen to { :ok , listener } = Midiex.Listener . start_link ( ports : Midiex . ports ( :input ) )","ref":"Midiex.Listener.html#start_link/1"},{"type":"function","title":"Midiex.Listener.subscribe/2","doc":"Subscribe to one or more MIDI input ports. This accepts both ports listed on your device %Midiex.MidiPort{direction: :input} and virtual ports %Midiex.VirtualMidiPort{} you've created. It accepts as it's second parameter either: a single MIDI input port a list of MIDI input ports :all atom which will stop all MIDI input ports subscribed to. Example # Subscribe to the input port of the Arturia KeyStep Pro keyboard keystep_in_port = Midiex . port ( &quot;KeyStep Pro&quot; , :input ) # Returns a list with matching port names, in this case: [ % Midiex.MidiPort { direction : :input , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.3139841870 . 4103995416.58432 &gt; } ] # Create and start a listener process { :ok , keyboard } = Midiex.Listener . start_link ( ) # Listen to MIDI messages from the keyboard Midiex.Listener . subscribe ( keyboard , keystep_in_port ) # Any keys you push on the keyboard will be listened to. Add one or more handlers with Midiex.Listener.add_handler/2 to process messages.","ref":"Midiex.Listener.html#subscribe/2"},{"type":"function","title":"Midiex.Listener.unsubscribe/2","doc":"Stops listening to the MIDI input port by unsubscribing to it. This accepts both ports listed on your device %Midiex.MidiPort{direction: :input} and virtual ports %Midiex.VirtualMidiPort{} you've created. It accepts as it's second parameter either: a single MIDI input port a list of MIDI input ports :all atom which will stop all MIDI input ports subscribed to. Important This stops the Rust OS thread from sending messages from that MIDI input port. If other Elixir processes have also subscribed to that port, they will also stop recieving messages.","ref":"Midiex.Listener.html#unsubscribe/2"},{"type":"module","title":"Midiex.Message","doc":"Convenience functions for creating binary MIDI messages. MIDI message functions So that you don't have to remember all the MIDI message codes, this library has a large range of message generating functions. Some common types are below: note_on/3 note_off/3 all_notes_off/1 sound_off/1 polyphonic_aftertouch/3 channel_aftertouch/3 control_change/3 program_change/2 pitch_bend/2 pan/2 volume/2 Various system messages are supported too, such as sysex/1 . Some functions have an optional high-resolution (14-bit) version. This is activated by providing the high_res: true option, e.g.: Midiex.Message.volume(16383, high_res: true) . You can learn more about the resolution of MIDI messages below. Notes Functions that take notes as a parameter can accept a number, string or atom note representation. For example, middle-C can be represented as 60 , &quot;C4&quot; or :C4 . Taking the note_on/1 function as an example, generating a &quot;note on&quot; MIDI message for middle-C &lt;&lt;144, 60, 127&gt;&gt; can be achieved in any of these ways: Message.note_on(60) Message.note_on(&quot;C4&quot;) Message.note_on(:C4) To get the MIDI number for a string or atom note representation, see note/1 . Example alias Midiex.Message , as : M # Connect to the synth, in this case the Arturia MicroFreak synth synth = Midiex . ports ( &quot;Arturia MicroFreak&quot; , :output ) |&gt; Midiex . open ( ) # Send the note on message, for D3 Midiex . send_msg ( synth , M . note_on ( :D3 ) ) # Wait 1 second :timer . sleep ( 1000 ) # Send the note off message, for D3 Midiex . send_msg ( synth , M . note_off ( :D3 ) ) About MIDI messages MIDI messages are in the format of: status_byte + note_number (0-127) + velocity (0-127) For example, taking the status byte for Note On which in HEX is 0x90 , and the note Middle C which is 60 and a maximum key velocity of 127, the MIDI message in binary format is: &lt;&lt;0x90, 60, 127&gt;&gt; Resolution (bits) Coarse resolution This type of message format is sometimes called 'coarse' or 7-bit MIDI, as it takes a maximum of 128 values only (from 0 to 127). High resolution However, 'high resolution' or 14-bit MIDI messages are possible, giving a maximum of 16,384 values (from 0 to 16,383). When working with controllers such as mod wheels, it allows for finer and smoother changes. To achieve this higher 14-bit resolution, the MIDI message combines two 7-bit values called: MSB (Most Significant Byte) , which is used for 'coarser' values. This is the bit which has the greatest effect. LSB (Least Significant Byte) , which is used for 'finer' values. In Elixir, We can unpack a 14-bit value into it's MSB and LSB bits using binary pattern matching, e.g.: &lt;&lt;msb::7, lsb::7&gt;&gt; = &lt;&lt;value::14&gt;&gt; The MSB and LSB values are sent as two different messages. Below is what it would look like for a Mod wheel which uses two control change command messages ( 0x01 and 0x21 ) to send the MSB and LSB respectively &lt;&lt; msb :: 7 , lsb :: 7 &gt;&gt; = &lt;&lt; mod_wheel_value :: 14 &gt;&gt; msb_binary = control_change ( 1 , msb ) lsb_binary = control_change ( 0x21 , lsb ) &lt;&lt; msb_binary :: binary , lsb_binary :: binary &gt;&gt; More information https://www.midi.org/midi-articles/about-midi-part-3-midi-messages","ref":"Midiex.Message.html"},{"type":"function","title":"Midiex.Message.active_sense/0","doc":"Creates a MIDI active sense message. Used to tell listening devices that the MIDI connection is still active.","ref":"Midiex.Message.html#active_sense/0"},{"type":"function","title":"Midiex.Message.all_notes_off/1","doc":"Creates an all notes off message. This mutes all sounding notes. The release time will still be maintained. Notes held by sustain will not turn off until sustain pedal is depressed.","ref":"Midiex.Message.html#all_notes_off/1"},{"type":"function","title":"Midiex.Message.balance/2","doc":"Controls the left and right balance, generally for stereo patches. A value of 64 equals the center. Values below 64 moves the sound to the left, and above to the right. Example Midiex.Message . balance ( 64 ) 14-bit version If you want a high-resolution balance message (e.g. in the range of 0-16383), you can pass the option high_res: true , e.g.: # Center channel (high res) Midiex.Message . balance ( 8191 , high_res : true ) # Returns: &lt;&lt; 176 , 8 , 63 , 176 , 40 , 127 &gt;&gt;","ref":"Midiex.Message.html#balance/2"},{"type":"function","title":"Midiex.Message.bank_select/2","doc":"Creates a bank select (also known as bank switch) message. Takes a bank number as it's first parameter, in the range of 0-16383. The number of banks is dependent on the device. A channel number can be provided as an option.","ref":"Midiex.Message.html#bank_select/2"},{"type":"function","title":"Midiex.Message.breath_controller/2","doc":"Creates a breath controller messsage. Breath controller messages were originally intended for use with a breath MIDI controller. Blowing harder into the breath controller would produce higher MIDI control values. Outside of breath control, is can be associated with aftertouch messages or used for modulation. Takes a number as it's first parameter, in the range of 0-16383. A channel number can be provided as an option.","ref":"Midiex.Message.html#breath_controller/2"},{"type":"function","title":"Midiex.Message.channel_aftertouch/3","doc":"Creates a channel aftertouch (also known as channel pressure) message. On a keyboard, an aftertouch message is sent by pressing down further on keys after it has already reached the bottom. Not all keyboards have aftertouch. With channel aftertouch, one of the following is used; Either the: average amount of pressure of all the keys held down; or the single greatest pressure value of all the current depressed keys. Therefore channel aftertouch is independent of which key or how many keys are held. polyphonic_aftertouch is specific to each key however. This function takes as it's parameters: note: a note as a string (e.g. &quot;C4&quot;), atom (e.g. :C4) or number (e.g. 60) as the first parameter pressure: a number between 0 and 127 representing the pressure on the key. By defaut 127 is used. The following option can also be passed: channel: the MIDI channel to which the message will be sent (there are 16 channels per MIDI device, in the range 0 to 15). By default channel 0 is used. Example alias Midiex.Message # Create a series of aftertouch messages from 0 to 127 for the note middle-C (C4) 0 .. 127 // 1 |&gt; Enum . map ( fn pressure -&gt; Message . channel_aftertouch ( :C4 , pressure : pressure ) end )","ref":"Midiex.Message.html#channel_aftertouch/3"},{"type":"function","title":"Midiex.Message.clock/0","doc":"Creates a MIDI clock message, used for clock synchronization. The MIDI clock message is a timing message that is sent at regular intervals to tell the listening devices where it is in terms of time.","ref":"Midiex.Message.html#clock/0"},{"type":"function","title":"Midiex.Message.control_change/3","doc":"Creates a MIDI CC or 'control change' message. MIDI Control Change messages are used to control functions in a synthesiser. Controllers include devices such as pedals, levers/sliders, wheels, switches and other control-oriented devices. This function takes as its parameters: contoller number: a number between 0-119. Controller numbers between 120-127 are reserved as &quot;Channel Mode Messages&quot;. value: depends on the control function, but usually is a a number between 0 and 127. See the MIDI 1.0 Control Change Messages Spec or consult the MIDI device manual for specific codes an values. The following option can be passed: channel: the MIDI channel to which the message will be sent (there are 16 channels per MIDI device, in the range 0 to 15). By default channel 0 is used. Example The MIDI CC message of 123 equates to &quot;All Notes Off&quot; (a Channel Mode Message), thus stopping all notes being played. # Create a &#39;all notes off&#39; CC message. Midiex . control_change ( 127 ) Reference See the official MIDI 1.0 Control Change Messages Spec .","ref":"Midiex.Message.html#control_change/3"},{"type":"function","title":"Midiex.Message.data_entry_msb/2","doc":"Creates an data entry message for MSB. Used to control the value for NRPN (Non-Registered Parameter Number) or RPN (Registered Parameter Number) parameters. Takes a number as it's first parameter, in the range of 0-16383. A channel number can be provided as an option.","ref":"Midiex.Message.html#data_entry_msb/2"},{"type":"function","title":"Midiex.Message.foot_controller/2","doc":"Creates a foot or pedal controller messsage. Takes a number as it's first parameter, in the range of 0-16383. A channel number can be provided as an option.","ref":"Midiex.Message.html#foot_controller/2"},{"type":"function","title":"Midiex.Message.local_control/2","doc":"Switches local control on or off by creating a local control message. An example of local control is that you may want the synthesizer to be played by means of its own keyboard, therefore setting Midiex.Message.local_control(true) . If you were controlling it from a PC only, and didn't want it to have local control, you could send the Midiex.Message.local_control(false) message. More information at: https://electronicmusic.fandom.com/wiki/Local_control","ref":"Midiex.Message.html#local_control/2"},{"type":"function","title":"Midiex.Message.mod_wheel/2","doc":"Creates a modulation (mod) wheel message. Modulation wheels are often used for vibrato effects (pitch, loudness, brighness), however what is modulated is based on the patch. Takes a number as it's first parameter, in the range of 0-16383. A channel number can be provided as an option.","ref":"Midiex.Message.html#mod_wheel/2"},{"type":"function","title":"Midiex.Message.mono_mode/3","doc":"Creates a message which will set the device to monophonic mode. Takes as it's parameters: true to set mono mode on, or false to switch on poly mode the number of channels, or 0 if the number of channels equals the number of voices in the receiver. Example # Set device to monophonic mode Midiex.Message . mono_mode ( ) # Returns: &lt;&lt; 176 , 126 , 0 &gt;&gt;","ref":"Midiex.Message.html#mono_mode/3"},{"type":"function","title":"Midiex.Message.note/1","doc":"Returns the MIDI numerical code for a note. Takes either a string or atom representation of a note as the first parameter. Example # Return the code for middle-C (also known as C4) Message . note ( :C4 ) Message . note ( &quot;C4&quot; ) Message . note ( :MiddleC ) Message . note ( &quot;MiddleC&quot; ) # These all return: 60 # Return the code for A-sharp 3 Message . note ( :As3 ) Message . note ( &quot;A#3&quot; ) # These both return: 58","ref":"Midiex.Message.html#note/1"},{"type":"function","title":"Midiex.Message.note_off/3","doc":"Creates a MIDI note-off message. Takes as it's parameters: note: a note as a string (e.g. &quot;C4&quot;), atom (e.g. :C4) or number (e.g. 60) as the first parameter velocity: a number between 0 and 127 representing how hard (or loud) a key was pressed. By defaut 127 is used. The following options can be passed: channel: the MIDI channel to which the message will be sent (there are 16 channels per MIDI device, in the range 0 to 15). By default channel 0 is used. All notes can be switched off with Message.all_notes_off/1 . Example # Note-off for middle-C Message . note_off ( :C4 ) # Returns: &lt;&lt;128, 60, 127&gt;&gt;","ref":"Midiex.Message.html#note_off/3"},{"type":"function","title":"Midiex.Message.note_on/3","doc":"Creates a MIDI note-on message. Takes as it's parameters: note: a note as a string (e.g. &quot;C4&quot;), atom (e.g. :C4) or number (e.g. 60) as the first parameter velocity: a number between 0 and 127 representing how hard (or loud) a key was pressed. By defaut 127 is used. The following options can be passed: channel: the MIDI channel to which the message will be sent (there are 16 channels per MIDI device, in the range 0 to 15). By default channel 0 is used. Note that MIDI channels are in the range 0 - 15. But in MIDI software and hardware it may be offset by +1, so MIDI channel 0 might be called MIDI channel 1 and so on to channel 16. Examples # Note-on message for middle-C Midiex.Message . note_on ( :C4 ) # Returns: &lt;&lt;144, 60, 127&gt;&gt; # Note-on message for middle-C on channel 2 with a velocity of 40 Midiex.Message . note_on ( :C4 , 40 , channel : 2 ) # Returns: &lt;&lt;146, 60, 40&gt;&gt; These can be sent to a connection using Midiex.send_msg/2 , for example: alias Midiex.Message Midiex . send_msg ( out_conn , Message . note_on ( :C4 , 40 , channel : 2 ) )","ref":"Midiex.Message.html#note_on/3"},{"type":"function","title":"Midiex.Message.omni_mode/2","doc":"Sets Omni mode on or off. The first parameter takes one of the following booleans: true to switch Omni mode on false to swithc Omni mode off","ref":"Midiex.Message.html#omni_mode/2"},{"type":"function","title":"Midiex.Message.pan/2","doc":"Change the panoramic (pan) of a channel. This shifts the sound from the left or right ear in when playing stereo. Values below 64 moves the sound to the left, and above to the right. Example # Pan to middle Midiex.Message . pan ( 64 ) 14-bit version If you want a high-resolution pan message (e.g. in the range of 0-16383), you can pass the option high_res: true . # Pan to middle (high res version) Midiex.Message . pan ( 8191 , high_res : true )","ref":"Midiex.Message.html#pan/2"},{"type":"function","title":"Midiex.Message.pitch_bend/2","doc":"Creates a pitch bend message, representing a change in pitch. Pitch bend change messages a usually sent from a keyboard with a pitch bend wheel or lever. Takes as it's first parameter, the pitch bend amount. The channel cane be passed as an option. The range of a pitch bend is as follows: 0-8191 represent negative bends, 8192 (Hex: 0x2000) is no bend and 8193-16383 are positive bends Example alias Midiex.Message , as : M # Play a note Midiex . send_msg ( piano , M . note_on ( :D3 ) ) # Bend up, then down, then back to center (8192) ( Enum . to_list ( 8193 .. 16383 // 1 ) ++ Enum . to_list ( 8191 .. 0 // - 1 ) ++ [ 8192 ] ) |&gt; Enum . each ( fn pitch -&gt; Midiex . send_msg ( piano , M . pitch_bend ( pitch ) ) end )","ref":"Midiex.Message.html#pitch_bend/2"},{"type":"function","title":"Midiex.Message.poly_mode/2","doc":"Creates a message which will set the device to polyphonic mode. Takes as it's parameters: true to set poly mode on, or false to switch on mono mode (or use mono_mode ) Example Midiex.Message . poly_mode ( ) # Returns &lt;&lt; 176 , 127 , 0 &gt;&gt;","ref":"Midiex.Message.html#poly_mode/2"},{"type":"function","title":"Midiex.Message.polyphonic_aftertouch/3","doc":"Creates a polyphonic aftertouch message. On a keyboard, polyphonic aftertouch (or key pressure) is message sent by pressing down further on a key after it has already reached the bottom. Not all keyboards have aftertouch. Note: polyphonic_aftertouch is specific to each key, where as channel_aftertouch is average amount of pressure applied to whichever keys are held down. The function takes as it's parameters: note: a note as a string (e.g. &quot;C4&quot;), atom (e.g. :C4) or number (e.g. 60) as the first parameter pressure: a number between 0 and 127 representing the pressure on the key. By defaut 127 is used. The following options can be passed: channel: the MIDI channel to which the message will be sent (there are 16 channels per MIDI device, in the range 0 to 15). By default channel 0 is used. Example alias Midiex.Message # Create a series of aftertouch messages from 0 to 127 for the note middle-C (C4) 0 .. 127 // 1 |&gt; Enum . map ( fn pressure -&gt; Message . polyphonic_aftertouch ( :C4 , pressure ) end )","ref":"Midiex.Message.html#polyphonic_aftertouch/3"},{"type":"function","title":"Midiex.Message.portamento/2","doc":"Creates a portamento (slide or glide) time messsage. Portamento is the rate to slide between 2 notes played in sequence, sliding the pitch up or down from one note to the next. Takes a number as it's first parameter, in the range of 0-16383. A channel number can be provided as an option.","ref":"Midiex.Message.html#portamento/2"},{"type":"function","title":"Midiex.Message.program_change/2","doc":"Creates a program change message, used select the instrument type to play sounds with or a different 'patch'. This function takes one data byte which specifies the new program number. To apply this to a particular channel, use the channel option, e.g.: Message . program_change ( 1 , channel : 3 )","ref":"Midiex.Message.html#program_change/2"},{"type":"function","title":"Midiex.Message.quarter_frame/1","doc":"Creates a MIDI quarter frame message, used to send timing information. Takes a single byte as the first parameter. Timing information is in the MIDI time code format, which is hours:minutes:seconds:frames. This follows the same timing information as standard SMPTE timecode . As MIDI send values in the range of 0-127, a single byte cannot carry the full time. For this reason, 8 quarter frame messages must be sent to piece together the current MIDI time. The timecode/1 function will automatically convert a hours:minutes:seconds:frames timecode string create the individual quarter frames.","ref":"Midiex.Message.html#quarter_frame/1"},{"type":"function","title":"Midiex.Message.reset/0","doc":"Creates a MIDI reset message. Various MIDI devices will interpret this message differently. Often it will cause a device to stop playing and set the song position to the beginning. The MIDI 1.0 specification says the following should occur when a reset message is sent: The modulation wheel, hold pedal, portamento pedal, sostenuto pedal and soft pedal are set to 0 The pitch wheel is set to center which is usually 64, but could be set to 0 The channel pressure and the key pressure are set to 0 Registered and nonregistered parameter numbers (98 to 101) are set to 127 Expression is set to 127.","ref":"Midiex.Message.html#reset/0"},{"type":"function","title":"Midiex.Message.reset_controllers/1","doc":"Creates a message that will reset all controllers to their default.","ref":"Midiex.Message.html#reset_controllers/1"},{"type":"function","title":"Midiex.Message.resume/0","doc":"Creates a MIDI continue message. Used to tell listening devices to resume playback of the current MIDI sequence.","ref":"Midiex.Message.html#resume/0"},{"type":"function","title":"Midiex.Message.sound_off/1","doc":"Creates an all sound off message. This mutes all sound regardless of release time or sustain.","ref":"Midiex.Message.html#sound_off/1"},{"type":"function","title":"Midiex.Message.start/0","doc":"Creates a MIDI start message. Used to tell listening devices to commence playback of the current MIDI sequence.","ref":"Midiex.Message.html#start/0"},{"type":"function","title":"Midiex.Message.stop/0","doc":"Creates a MIDI stop message. Used to tell listening devices to stop playing the current MIDI sequence.","ref":"Midiex.Message.html#stop/0"},{"type":"function","title":"Midiex.Message.sysex/2","doc":"Creates a system exclusive message, also known as a SysEx message. This function takes the following parameters: SysEx ID number : A code representing the device manufacturer, see the offical list of manurfacturer IDs Data : series of hex data bytes representing the message body. The hex data bytes values are between 0x00 and 0x7F (0 and 127). About SysEx SysEx messages can contain any number of hexadecimal bytes. They the message data is specific to each manufacturers device and include the manufacturer's identification code. SysEx messages are wrapped in a start (0xF0) and end (0xF7) byte, e.g.: &lt;&lt;0xF0, id_number, data, 0xF7&gt;&gt; Example # Send data to a Roland device # Roland uses the ID of 0x41 (or you can pass the integer of 65) Midiex.Message . sysex ( 0x41 , &lt;&lt; 0x01 , 0x34 &gt;&gt; ) # Returns &lt;&lt;240, 65, 1, 52, 247&gt;&gt; # You can pass integer values instead Midiex.Message . sysex ( 65 , &lt;&lt; 1 , 52 &gt;&gt; ) # Returns &lt;&lt;240, 65, 1, 52, 247&gt;&gt;","ref":"Midiex.Message.html#sysex/2"},{"type":"function","title":"Midiex.Message.timecode/1","doc":"Creates a eight MIDI quarter frame messages to represent a single hours:minutes:seconds:frames timecode string. Example Midiex.Message . timecode ( &quot;01:00:00:00&quot; ) # Returns # &lt;&lt;241, 0, 241, 16, 241, 32, 241, 48, 241, 64, 241, 80, 241, 97, 241, 112&gt;&gt;","ref":"Midiex.Message.html#timecode/1"},{"type":"function","title":"Midiex.Message.volume/2","doc":"Creates a volume messsage. This was formally called 'Main Volume' in the MIDI 1.0 spec. Takes a number as it's first parameter, in the range of 0-127. For example, to set to maximum volume: Midiex.Message . volume ( 127 ) # Returns: &lt;&lt; 176 , 7 , 127 &gt;&gt; 14-bit version If you want a high-resolution volume message (e.g. in the range of 0-16383), you can pass the option high_res: true , e.g.: Midiex.Message . volume ( 16383 , high_res : true ) # Returns: &lt;&lt; 176 , 7 , 127 , 176 , 39 , 127 &gt;&gt; A channel number can be provided as an option.","ref":"Midiex.Message.html#volume/2"},{"type":"module","title":"Midiex.Notifier","doc":"GenServer for subscribing and responding to MIDI notifications on supported systems. This is currently implemented for Mac only. How this works On Mac, a callback function needs to be created to specially handle MIDI notification messages, such as when a device has been physically plugged ( :added ) or unplugged ( :removed ). This callback is implemented in the Rust side of this library using coremidi . The notifications will be delivered on MacOS to a Rust thread with the specific 'run loop' (using CFRunLoop from the core_foundation Rust library) that was created when the Midiex.notifications/0 function was first called. This function is called automatically when this GenServer is started. Any ( :added ) or ( :removed ) notifications will be sent to this GenServer from the Rust thread. The Midiex.Notifier GenServer then forwards these notifications to any handler functions added in with the add_handler/2 function or passed to it through the start/1 function. Example # Start the Notifier GenServer { :ok , pid } = Midiex.Notifier . start_link ( ) # Add a simple handler callback function. This will just inspect the notification. Midiex.Notifier . add_handler ( pid , fn msg -&gt; IO . inspect ( msg ) end ) # KeyStep Pro keyboard has been hot-plugged into the Mac: % Midiex.MidiNotification { notification_type : :added , parent_name : &quot;KeyStep Pro&quot; , parent_id : 1384647386 , parent_type : :entity , name : &quot;KeyStep Pro&quot; , native_id : 493507367 , direction : :input } % Midiex.MidiNotification { notification_type : :added , parent_name : &quot;KeyStep Pro&quot; , parent_id : 1384647386 , parent_type : :entity , name : &quot;KeyStep Pro&quot; , native_id : 2688501783 , direction : :output } # KeyStep Pro keyboard has been unplugged from the Mac: % Midiex.MidiNotification { notification_type : :removed , parent_name : &quot;KeyStep Pro&quot; , parent_id : 1384647386 , parent_type : :entity , name : &quot;KeyStep Pro&quot; , native_id : 493507367 , direction : :input } % Midiex.MidiNotification { notification_type : :removed , parent_name : &quot;KeyStep Pro&quot; , parent_id : 1384647386 , parent_type : :entity , name : &quot;KeyStep Pro&quot; , native_id : 2688501783 , direction : :output }","ref":"Midiex.Notifier.html"},{"type":"function","title":"Midiex.Notifier.add_handler/2","doc":"Add one or more callback function(s) which will recieve and handle MIDI notifications. The first parameter of the callback function will be used to recieve notification messages. Example # Start the Notifier GenServer { :ok , pid } = Midiex.Notifier . start_link ( ) # Add a simple handler callback function. This will just inspect the notification. Midiex.Notifier . add_handler ( pid , fn msg -&gt; IO . inspect ( msg ) end )","ref":"Midiex.Notifier.html#add_handler/2"},{"type":"function","title":"Midiex.Notifier.child_spec/1","doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Midiex.Notifier.html#child_spec/1"},{"type":"function","title":"Midiex.Notifier.clear_handlers/1","doc":"Clears all handlers.","ref":"Midiex.Notifier.html#clear_handlers/1"},{"type":"function","title":"Midiex.Notifier.get_state/1","doc":"Gets the servers state, returns %Midiex.Listener{} struct.","ref":"Midiex.Notifier.html#get_state/1"},{"type":"function","title":"Midiex.Notifier.new/1","doc":"Creates a new %Midiex.Notifier{} struct. Takes an optional keyword list as the first parameter which can be used to populate individual struct keys. The struct holds the following key-values: :callback which holds a list of functions called when a message is received for an input port. The callback must be of single arity and take it's first parameter a message. See add_handler/2 for an example.","ref":"Midiex.Notifier.html#new/1"},{"type":"function","title":"Midiex.Notifier.start_link/1","doc":"Start the Midiex.Notifier GenServer. Takes an optional keyword list as the first parameter which can be used to populate individual %Midiex.Notifier{} struct keys. See new/1 for informaton.","ref":"Midiex.Notifier.html#start_link/1"},{"type":"module","title":"Midiex.MidiMessage","doc":"A struct representing MIDI messages recieved from MIDI inputs. These are recieved via the Midiex.subscribe() function or from the Midiex.Listener GenServer. The keys are as follows: port: which is the input port ( %Midiex.MidiPort{} ) that sent the message data: the MIDI message data, usually in the form of a three item list, e.g. [153, 60, 70] timestamp: from the midir docs : &quot;a timestamp (in microseconds) designating the time since some unspecified point in the past (which will not change during the lifetime of an input connection)&quot;. Example messages % Midiex.MidiMessage { port : % Midiex.MidiPort { direction : :input , name : &quot;KeyStep Pro&quot; , num : 2 , port_ref : # Reference &lt; 0.2327272197 . 1194197016.109029 &gt; } , data : [ 153 , 60 , 70 ] , timestamp : 283146647865 } % Midiex.MidiMessage { port : % Midiex.MidiPort { direction : :input , name : &quot;Arturia MicroFreak&quot; , num : 1 , port_ref : # Reference &lt; 0.2327272197 . 1194197016.109028 &gt; } , data : [ 128 , 53 , 33 ] , timestamp : 283145644340 } % Midiex.MidiMessage { port : % Midiex.MidiPort { direction : :input , name : &quot;Arturia DrumBrute Impact&quot; , num : 0 , port_ref : # Reference &lt; 0.2327272197 . 1194197016.109027 &gt; } , data : [ 153 , 36 , 4 ] , timestamp : 283147540161 } Other examples See Midiex.Listener for examples of subscribing to MIDI messages and adding your own callback functions to process them.","ref":"Midiex.MidiMessage.html"},{"type":"module","title":"Midiex.MidiNotification","doc":"A struct representing notifications of MIDI changes. This is currently only implemented on MacOS and is capturing added or removed messages only (e.g. a device or port has been added or removed). An example use of this is for hot swapping of devices, responding to if a device has been added or removed. The main fields are: notification_type: which is of type :added or :removed name: which is the same as the port name used in %Midiex.MidiPort{} direction: which is the same as the port direction used in %Midiex.MidiPort{} . Additionally, the following fields have been included from coreaudio (MacOS): parent_name: which may or may not be the same as the name: field above parent_type: the 'parent type' reported by coreaudio, often :entity parent_id: the unique numerical ID reported by coreaudio for the parent native_id: the unique numerical ID reported by coreaudio for the port Example # KeyStep Pro keyboard has been hot-plugged into the Mac: % Midiex.MidiNotification { notification_type : :added , parent_name : &quot;KeyStep Pro&quot; , parent_id : 1384647386 , parent_type : :entity , name : &quot;KeyStep Pro&quot; , native_id : 493507367 , direction : :input } % Midiex.MidiNotification { notification_type : :added , parent_name : &quot;KeyStep Pro&quot; , parent_id : 1384647386 , parent_type : :entity , name : &quot;KeyStep Pro&quot; , native_id : 2688501783 , direction : :output } # KeyStep Pro keyboard has been unplugged into the Mac: % Midiex.MidiNotification { notification_type : :removed , parent_name : &quot;KeyStep Pro&quot; , parent_id : 1384647386 , parent_type : :entity , name : &quot;KeyStep Pro&quot; , native_id : 493507367 , direction : :input } % Midiex.MidiNotification { notification_type : :removed , parent_name : &quot;KeyStep Pro&quot; , parent_id : 1384647386 , parent_type : :entity , name : &quot;KeyStep Pro&quot; , native_id : 2688501783 , direction : :output }","ref":"Midiex.MidiNotification.html"},{"type":"module","title":"Midiex.MidiPort","doc":"A struct representing a MIDI port. The keys are as follows: direction which is an atom of value :input or :output (for input or output port) name which is a string the backend reported as the name of the port. With MIDI hardware, this is often the name of the device. num an integer index representing the port starting at 0. Both input and output ports will start with 0. port_ref a reference (e.g. #Reference&lt;0.2239960018.1937899544.176288&gt; ) to the port object in midir (Rust). Notes from midir How a port is identified internally is backend-dependent. If the backend allows it, port objects remain valid when other ports in the system change (i.e. it is not just an index). MidiInputPort: https://docs.rs/midir/latest/midir/struct.MidiInputPort.html MidiOutputPort: https://docs.rs/midir/latest/midir/struct.MidiInputPort.html Example Midiex . ports ( ) This will return MIDI ports available on your system, for example: [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.2239960018 . 1937899544.176288 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.2239960018 . 1937899544.176289 &gt; } ]","ref":"Midiex.MidiPort.html"},{"type":"module","title":"Midiex.OutConn","doc":"A struct representing an open connection to a MIDI output port. The keys are as follows: conn_ref the reference (e.g. #Reference&lt;0.2239960018.1937899544.176288&gt; ) to the connection object in midir (Rust). name a string containing the name of the port this connection is to port_num a integer representing the index of the output port. Documentation from midir See MidiOutputConnection at: https://docs.rs/midir/latest/midir/struct.MidiOutputConnection.html Example # Pass a port from taken from Midiex.ports(:output) # e.g. port = Midiex.ports(:output) |&gt; List.first() port = % Midiex.MidiPort { direction : :output , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.3876911033 . 1674706968.249863 &gt; } output_conn = Midiex . open ( port ) output_conn will look something like this: % Midiex.OutConn { conn_ref : # Reference &lt; 0.3876911033 . 1674706945.249916 &gt; , name : &quot;IAC Driver Bus 1&quot; , port_num : 0 } An output port can be closed as follows: Midiex . close ( output_conn ) # :ok is returned if successful","ref":"Midiex.OutConn.html"},{"type":"module","title":"Midiex.VirtualMidiPort","doc":"A struct representing a virtual MIDI port. Currently this is only used for :input ports. When a viritual output port is created a Midiex.OutConn struct is returned instead. Note that viritual ports are only available on platforms that them (currently every platform but Windows). The keys of the struct are as follows: direction which is an atom currently of value :input name which is a string containing the name of the port num the port number Example Virtual input port # Create a virtual MIDI input by giving it a name. MIDIex will also assign it an input port number (`num`). my_virtual_in = Midiex . create_virtual_input ( &quot;My Virtual Input&quot; ) # This will return a VirtualMidiPort struct in the following format # %Midiex.VirtualMidiPort{direction: :input, name: &quot;My Virtual Input&quot;, num: 1} More information To create a virtual input port see Midiex.create_virtual_input/1","ref":"Midiex.VirtualMidiPort.html"},{"type":"extras","title":"README","doc":"Midiex overview Midiex is a cross-platform, real-time MIDI processing library in Elixir.","ref":"readme.html"},{"type":"extras","title":"README - midir Rust library","doc":"Using Rustler , Midiex wraps the excellent midir Rust library. Midir support a range of platforms and backends, such as: ALSA (Linux) WinMM (Windows) CoreMIDI (MacOS, iOS) WinRT (Windows 8+), Jack (Linux, macOS) Using WinRT or Jack requires special feature flags enabled. See the midir GitHub and create docs for more details. The hot-plug support of MIDI devices on MacOS is made possible with with the Rust coremidi library.","ref":"readme.html#midir-rust-library"},{"type":"extras","title":"README - Status","doc":"This library is currently under active development and it’s API is likely to change. It's been tested on MacOS only.","ref":"readme.html#status"},{"type":"extras","title":"README - API","doc":"At it's most basic level, the core functions of Midiex are for: listing or counting MIDI ports availble (for example, a keyboard or synth) opening or closing connections to MIDI ports sending or receiving messages to and from connections creating virtual output connections so your Elixir application appears as a MIDI input port on the host OS. creating virtual input connections so your Elixir application appears as a MIDI output port on the host OS.","ref":"readme.html#api"},{"type":"extras","title":"README - Feature support","doc":"Not all midir features have been wrapped and some features are backend specific: Virtual output connection : currently on every platform but Windows Virtual input connection : currently on every platform but Windows Notification messages and hot-plug support : currently implemented on MacOS (e.g. to receive notifications when a device has been plugged in or removed).","ref":"readme.html#feature-support"},{"type":"extras","title":"README - MIDI messages","doc":"MIDI messages are in binary format. They're usually in the format of one status byte followed by one or two data bytes. For example, the status byte for 'Note On' is 0x90 in HEX format. The data byte representing the note Middle C is 60 . The data byte representing velocity (i.e. how hard the key was struck when the note was played) is an integer in the range 0 - 127 where 127 is the loudest. Putting that together, the message to play Middle C at a velocity of 127 is: &lt;&lt;0x90, 60, 127&gt;&gt; You can stop the same note from playing by sending the 'Note Off' status byte 0x80 , which would make the message: &lt;&lt;0x80, 60, 127&gt;&gt; . For more information on MIDI messages, see the offical MIDI Assocations Specifications , Expanded MIDI 1.0 message list or the various articles online such as this one .","ref":"readme.html#midi-messages"},{"type":"extras","title":"README - Example","doc":"# List MIDI ports Midiex . ports ( ) # Create a virtual output connection piano = Midiex . create_virtual_output ( &quot;piano&quot; ) # Returns an output connection: # %Midiex.OutConn{ # conn_ref: #Reference&lt;0.1633267383.3718381569.210768&gt;, # name: &quot;piano&quot;, # port_num: 0 # } # Send to MIDI messages to a connection note_on = &lt;&lt; 0x90 , 60 , 127 &gt;&gt; note_off = &lt;&lt; 0x80 , 60 , 127 &gt;&gt; Midiex . send_msg ( piano , note_on ) :timer . sleep ( 3000 ) # wait three seconds Midiex . send_msg ( piano , note_off )","ref":"readme.html#example"},{"type":"extras","title":"README - Getting started","doc":"Adding it to your Elixir project The package can be installed by adding midiex to your list of dependencies in mix.exs: def deps do [ { :midiex , &quot;~&gt; 0.5.0&quot; } ] End Using within LiveBook and IEx Mix . install ( [ { :midiex , &quot;~&gt; 0.5.0&quot; } ] ) LiveBook tour Also see the introductory tour in LiveBook at /livebook/midiex_notebook.livemd .","ref":"readme.html#getting-started"},{"type":"extras","title":"README - Documentation","doc":"The docs can be found at https://hexdocs.pm/midiex .","ref":"readme.html#documentation"},{"type":"extras","title":"MIDIex notebook","doc":"Mix . install ( [ { :midiex , github : &quot;haubie/midiex&quot; } ] ) * Getting midiex ( https :// github . com / haubie / midiex . git ) remote : Enumerating objects : 717 , done . remote : Counting objects : 100 % ( 174 / 174 ) , done . remote : Compressing objects : 100 % ( 87 / 87 ) , done . remote : Total 717 ( delta 79 ) , reused 139 ( delta 52 ) , pack - reused 543 origin / HEAD set to main Resolving Hex dependencies ... Resolution completed in 0.257 s New : jason 1.4 . 1 rustler 0.26 . 0 toml 0.7 . 0 * Getting rustler ( Hex package ) * Getting jason ( Hex package ) * Getting toml ( Hex package ) == &gt; toml Compiling 10 files ( . ex ) Generated toml app == &gt; jason Compiling 10 files ( . ex ) Generated jason app == &gt; rustler Compiling 7 files ( . ex ) Generated rustler app == &gt; midiex Compiling 10 files ( . ex ) Compiling crate midiex in release mode ( native / midiex ) Compiling memchr v2 . 5.0 Compiling core - foundation - sys v0 . 8.3 Compiling libc v0 . 2.138 Compiling proc - macro2 v1 . 0.63 Compiling unicode - ident v1 . 0.5 Compiling coremidi - sys v3 . 1.0 Compiling quote v1 . 0.28 Compiling regex - syntax v0 . 6.28 Compiling void v1 . 0.2 Compiling heck v0 . 4.0 Compiling rustler v0 . 29.0 Compiling lazy_static v1 . 4.0 Compiling unreachable v1 . 0.0 Compiling block v0 . 1.6 Compiling bitflags v1 . 3.2 Compiling aho - corasick v0 . 7.20 Compiling regex v1 . 7.0 Compiling rustler_sys v2 . 3.0 Compiling core - foundation v0 . 9.3 Compiling coremidi v0 . 6.0 Compiling coremidi v0 . 7.0 Compiling midir v0 . 9.1 Compiling syn v2 . 0.22 Compiling rustler_codegen v0 . 29.0 Compiling midiex v0 . 1.0 ( / Users / haubie / Library / Caches / mix / installs / elixir - 1.15 . 2 - erts - 14.0 . 2 / 56 c7aef4f3fb67cf9f7eb75d4a18b401 / deps / midiex / native / midiex ) Finished release [ optimized ] target ( s ) in 6.13 s Generated midiex app :ok","ref":"midiex_notebook.html"},{"type":"extras","title":"MIDIex notebook - Introduction","doc":"Learning objectives This will get you started with Midiex. By the end of this Livebook you'll be able to: Find and connect to MIDI ports on your system Create virtual ports (on supported systems, like MacOS and Linux) Send and recieve messages. Setup Just to make our code a bit more compact when experimenting with live-music coding, we'll alias the Midiex.Message module as M . Midiex function names have been kept compact as possible with live-music coding in mind. alias Midiex.Listener alias Midiex.Message , as : M Midiex.Message","ref":"midiex_notebook.html#introduction"},{"type":"extras","title":"MIDIex notebook - MIDI concepts","doc":"Skip this section you're familiar with MIDI concepts and want to start playing with the library. At it's most basic, MIDI consists of: Ports , which represent input or output connections to MIDI hardware or software. You can recieve MIDI messages from a MIDI input, or send MIDI messages to a MIDI output. Connections , just like with all IO operations, you'll need to make a connection with a MIDI port to send or recieve messages to it. Messages , which are usually music related, such switching a note on or off.","ref":"midiex_notebook.html#midi-concepts"},{"type":"extras","title":"MIDIex notebook - Finding devices (ports)","doc":"Note that on Apple Mac, you may wish to call Midiex.hotplug() first so that Midiex will be able to see devices plugged in or removed. Midiex . hotplug ( ) :ok How many ports are there? Midiex . port_count ( ) %{ input : 4 , output : 3 } List devices (ports) ports = Midiex . ports ( ) [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1170 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 2&quot; , num : 1 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1171 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 3&quot; , num : 2 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1172 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;My Out&quot; , num : 3 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1173 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1174 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;IAC Driver Bus 2&quot; , num : 1 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1175 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;IAC Driver Bus 3&quot; , num : 2 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1176 &gt; } ] Filter to show input or output ports Midiex . ports ( :input ) [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1177 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 2&quot; , num : 1 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1178 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 3&quot; , num : 2 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1179 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;My Out&quot; , num : 3 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1180 &gt; } ] Midiex . ports ( :output ) [ % Midiex.MidiPort { direction : :output , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1188 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;IAC Driver Bus 2&quot; , num : 1 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1189 &gt; } , % Midiex.MidiPort { direction : :output , name : &quot;IAC Driver Bus 3&quot; , num : 2 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1190 &gt; } ] Filtering ports by name You can include a regular expression as the first parameter to search for matching ports. For example, to get the output ports from any Arturia device plugged into your system, you could do the following: Midiex . ports ( ~r/Arturia/ , :output ) [ ] If you know the name of the port, you can pass it as a string as the first parameter: Midiex . ports ( &quot;IAC Driver Bus 1&quot; , :input ) [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1198 &gt; } ]","ref":"midiex_notebook.html#finding-devices-ports"},{"type":"extras","title":"MIDIex notebook - Connecting to output devices","doc":"If you wanted to connect to the first Arturia output port on your system you could: # Get the port out_port = Midiex . ports ( ~r/Arturia/ , :output ) |&gt; List . first ( ) nil # Make a connection out_conn = Midiex . open ( out_port ) You can now send a message to the device via the output connection, for example: Midiex . send_msg ( out_conn , &lt;&lt; 0x90 , 60 , 127 &gt;&gt; ) error : undefined variable &quot;out_conn&quot; livebook / midiex_notebook . livemd #cell:752gysgdtdthfmkq2ehlkt5iw5bnzkyw:1","ref":"midiex_notebook.html#connecting-to-output-devices"},{"type":"extras","title":"MIDIex notebook - Virtual devices","doc":"A virtual output creates an output connection you can send messages to, but to other devices on your system, it wll appear as a MIDI input connection they can consume. For example, you might have a software synth installed on your PC that can consume these MIDI messages. virtual_conn = Midiex . create_virtual_output ( &quot;My Virtual Connection&quot; ) % Midiex.OutConn { conn_ref : # Reference &lt; 0.34604515 . 3318349831.2664 &gt; , name : &quot;My Virtual Connection&quot; , port_num : 4 } Note that although you've created this virtual output, on your system it will appear as an import port to be discoverable by other MIDI software or devices. If you call Midiex.ports/1 you'll see it as an input: ports = Midiex . ports ( :input ) [ % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 1&quot; , num : 0 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1212 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 2&quot; , num : 1 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1213 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;IAC Driver Bus 3&quot; , num : 2 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1214 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;My Out&quot; , num : 3 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1215 &gt; } , % Midiex.MidiPort { direction : :input , name : &quot;My Virtual Connection&quot; , num : 4 , port_ref : # Reference &lt; 0.34604515 . 3318349847.1216 &gt; } ]","ref":"midiex_notebook.html#virtual-devices"},{"type":"extras","title":"License (MIT)","doc":"MIT License Copyright (c) 2023 David Haubenschild. Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}]